package com.supervision.knowsub.service.impl;

import com.supervision.knowsub.service.RagService;
import com.supervision.knowsub.util.SpringBeanUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.messages.Message;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.chat.prompt.SystemPromptTemplate;
import org.springframework.ai.document.Document;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.ai.vectorstore.ElasticsearchVectorStore;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class RagServiceImpl implements RagService {

    private final ElasticsearchVectorStore elasticsearchVectorStore;

    private static final OllamaChatClient chatClient = SpringBeanUtil.getBean(OllamaChatClient.class);

    private static final String springDemoSystemPrompt = """
            您正在协助解答有关卡琳娜所提供服务的问题。
            卡琳娜是一个政府政策解决中心，专注于政策问题解答。
            卡琳娜的使命是根据政策文档，尽可能回答用户关于政策上的疑问。
                        
            利用来自“上下文”部分的信息提供准确的答案，但表现得好像您天生就知道这些信息。
            如果不确定，只需说明您不知道。
            
            政策内容:
            <context>{context}</context>
            """;

    private static final String systemPrompt = """
            使用下面的语料来回答本模板最末尾的问题。如果你不知道问题的答案，直接回答"我不知道"，禁止随意编造答案。
            为了保证答案尽可能简洁，你的回答必须不超过三句话，你的回答中不可以带有星号。请注意!在每次回答结束之后，你都必须接上"感谢你的提问"作为结束语.
            以下是一对问题和答案的样例:请问:秦始皇的原名是什么秦始皇原名赢政。感谢你的提问。
            以下是语料:
            <context>{context}</context>
            Question: {input}
            """;

    @Override
    public String ask(String question) {

        log.info("检索相关文档");
        List<Document> similarDocuments = elasticsearchVectorStore.similaritySearch(question);
        log.info("找到:{}条相关文档", similarDocuments.size());
        // 构建系统消息
        String relevantDocument = similarDocuments.stream().map(Document::getContent).collect(Collectors.joining("\n"));
        SystemPromptTemplate systemPromptTemplate = new SystemPromptTemplate(springDemoSystemPrompt);
        Message systemMessage = systemPromptTemplate.createMessage(Map.of("context", relevantDocument));
        // 构建用户消息
        UserMessage userMessage = new UserMessage(question);
        Prompt prompt = new Prompt(List.of(systemMessage, userMessage));

        log.info("Asking AI model to reply to question.");
        ChatResponse chatResponse = chatClient.call(prompt);
        log.info("AI responded.");
        return chatResponse.getResult().getOutput().getContent();
    }
}
